# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# NOTE! THIS FILE IS AUTOMATICALLY GENERATED AND WILL BE OVERWRITTEN!
#
# IF YOU WANT TO MODIFY THIS FILE, YOU SHOULD MODIFY THE TEMPLATE
# `get_provider_info_TEMPLATE.py.jinja2` IN the `dev/breeze/src/airflow_breeze/templates` DIRECTORY


def get_provider_info():
    return {
        "package-name": "apache-airflow-providers-alibaba",
        "name": "Alibaba",
        "description": "Alibaba Cloud integration (including `Alibaba Cloud <https://www.alibabacloud.com/>`__).\n",
        "integrations": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "external-doc-url": "https://www.alibabacloud.com/help/product/31815.htm",
                "logo": "/docs/integration-logos/alibabacloud-oss.png",
                "how-to-guide": ["/docs/apache-airflow-providers-alibaba/operators/oss.rst"],
                "tags": ["alibaba"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "external-doc-url": "https://www.alibabacloud.com/help/en/analyticdb-for-mysql/latest/spark-developerment",
                "how-to-guide": ["/docs/apache-airflow-providers-alibaba/operators/analyticdb_spark.rst"],
                "tags": ["alibaba"],
            },
        ],
        "operators": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "python-modules": ["airflow.providers.alibaba.cloud.operators.oss"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "python-modules": ["airflow.providers.alibaba.cloud.operators.analyticdb_spark"],
            },
            {
                "integration-name": "Alibaba Cloud MaxCompute",
                "python-modules": ["airflow.providers.alibaba.cloud.operators.maxcompute"],
            },
        ],
        "sensors": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "python-modules": ["airflow.providers.alibaba.cloud.sensors.oss_key"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "python-modules": ["airflow.providers.alibaba.cloud.sensors.analyticdb_spark"],
            },
        ],
        "hooks": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.oss"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.analyticdb_spark"],
            },
            {
                "integration-name": "Alibaba Cloud",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.base_alibaba"],
            },
            {
                "integration-name": "Alibaba Cloud MaxCompute",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.maxcompute"],
            },
        ],
        "connection-types": [
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.oss.OSSHook",
                "connection-type": "oss",
            },
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.analyticdb_spark.AnalyticDBSparkHook",
                "connection-type": "adb_spark",
            },
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.base_alibaba.AlibabaBaseHook",
                "connection-type": "alibaba_cloud",
            },
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.maxcompute.MaxComputeHook",
                "connection-type": "maxcompute",
            },
        ],
        "logging": ["airflow.providers.alibaba.cloud.log.oss_task_handler.OSSTaskHandler"],
        "extra-links": ["airflow.providers.alibaba.cloud.links.maxcompute.MaxComputeLogViewLink"],
    }
